@@ -4,12 +4,14 @@ from typing import Literal, Optional, Union
 import numpy as np
 from rich.console import Console
 from sentence_transformers import SentenceTransformer
+from sklearn.base import TransformerMixin
 from sklearn.feature_extraction.text import CountVectorizer
 from sklearn.mixture import BayesianGaussianMixture, GaussianMixture
+from sklearn.pipeline import Pipeline, make_pipeline
 
 from turftopic.base import ContextualModel, Encoder
-from turftopic.dynamic import DynamicTopicModel, bin_timestamps
-from turftopic.soft_ctf_idf import soft_ctf_idf
+from turftopic.dynamic import DynamicTopicModel
+from turftopic.feature_importance import soft_ctf_idf
 from turftopic.vectorizer import default_vectorizer
 
 
@@ -44,6 +46,16 @@ class GMM(ContextualModel, DynamicTopicModel):
         Concentration parameter of the symmetric prior.
         By default 1/n_components is used.
         Ignored when weight_prior is None.
+    dimensionality_reduction: TransformerMixin, default None
+        Optional dimensionality reduction step before GMM is run.
+        This is recommended for very large datasets with high dimensionality,
+        as the number of parameters grows vast in the model otherwise.
+        We recommend using PCA, as it is a linear solution, and will likely
+        result in Gaussian components.
+        For even larger datasets you can use IncrementalPCA to reduce
+        memory load.
+    random_state: int, default None
+        Random state to use so that results are exactly reproducible.
 
     Attributes
     ----------
@@ -58,13 +70,16 @@ class GMM(ContextualModel, DynamicTopicModel):
             Encoder, str
         ] = "sentence-transformers/all-MiniLM-L6-v2",
         vectorizer: Optional[CountVectorizer] = None,
+        dimensionality_reduction: Optional[TransformerMixin] = None,
         weight_prior: Literal["dirichlet", "dirichlet_process", None] = None,
         gamma: Optional[float] = None,
+        random_state: Optional[int] = None,
     ):
         self.n_components = n_components
         self.encoder = encoder
         self.weight_prior = weight_prior
         self.gamma = gamma
+        self.random_state = random_state
         if isinstance(encoder, str):
             self.encoder_ = SentenceTransformer(encoder)
         else:
@@ -73,17 +88,26 @@ class GMM(ContextualModel, DynamicTopicModel):
             self.vectorizer = default_vectorizer()
         else:
             self.vectorizer = vectorizer
+        self.dimensionality_reduction = dimensionality_reduction
         if self.weight_prior is not None:
-            self.gmm_ = BayesianGaussianMixture(
+            mixture = BayesianGaussianMixture(
                 n_components=n_components,
-                weight_concentration_prior_type="dirichlet_distribution"
-                if self.weight_prior == "dirichlet"
-                else "dirichlet_process",
+                weight_concentration_prior_type=(
+                    "dirichlet_distribution"
+                    if self.weight_prior == "dirichlet"
+                    else "dirichlet_process"
+                ),
                 weight_concentration_prior=gamma,
+                random_state=self.random_state,
             )
         else:
-            self.gmm_ = GaussianMixture(n_components)
-        self.components_ = None
+            mixture = GaussianMixture(
+                n_components, random_state=self.random_state
+            )
+        if dimensionality_reduction is not None:
+            self.gmm_ = make_pipeline(dimensionality_reduction, mixture)
+        else:
+            self.gmm_ = mixture
 
     def fit_transform(
         self, raw_documents, y=None, embeddings: Optional[np.ndarray] = None
@@ -105,10 +129,17 @@ class GMM(ContextualModel, DynamicTopicModel):
             self.components_ = soft_ctf_idf(
                 document_topic_matrix, document_term_matrix
             )
-            self.weights_ = self.gmm_.weights_
             console.log("Model fitting done.")
         return document_topic_matrix
 
+    @property
+    def weights_(self) -> np.ndarray:
+        if isinstance(self.gmm_, Pipeline):
+            model = self.gmm_.steps[-1][1]
+        else:
+            model = self.gmm_
+        return model.weights_
+
     def transform(
         self, raw_documents, embeddings: Optional[np.ndarray] = None
     ) -> np.ndarray:
@@ -137,8 +168,10 @@ class GMM(ContextualModel, DynamicTopicModel):
         embeddings: Optional[np.ndarray] = None,
         bins: Union[int, list[datetime]] = 10,
     ):
-        time_labels, self.time_bin_edges = bin_timestamps(timestamps, bins)
-        if self.components_ is not None:
+        time_labels, self.time_bin_edges = self.bin_timestamps(
+            timestamps, bins
+        )
+        if hasattr(self, "components_"):
             doc_topic_matrix = self.transform(
                 raw_documents, embeddings=embeddings
             )
@@ -147,9 +180,13 @@ class GMM(ContextualModel, DynamicTopicModel):
                 raw_documents, embeddings=embeddings
             )
         document_term_matrix = self.vectorizer.transform(raw_documents)
-        temporal_components = []
-        temporal_importances = []
-        for i_timebin in np.arange(len(self.time_bin_edges) - 1):
+        n_comp, n_vocab = self.components_.shape
+        n_bins = len(self.time_bin_edges) - 1
+        self.temporal_components_ = np.zeros(
+            (n_bins, n_comp, n_vocab), dtype=document_term_matrix.dtype
+        )
+        self.temporal_importance_ = np.zeros((n_bins, n_comp))
+        for i_timebin in np.unique(time_labels):
             topic_importances = doc_topic_matrix[time_labels == i_timebin].sum(
                 axis=0
             )
@@ -159,8 +196,6 @@ class GMM(ContextualModel, DynamicTopicModel):
                 doc_topic_matrix[time_labels == i_timebin],
                 document_term_matrix[time_labels == i_timebin],  # type: ignore
             )
-            temporal_components.append(components)
-            temporal_importances.append(topic_importances)
-        self.temporal_components_ = np.stack(temporal_components)
-        self.temporal_importance_ = np.stack(temporal_importances)
+            self.temporal_components_[i_timebin] = components
+            self.temporal_importance_[i_timebin] = topic_importances
         return doc_topic_matrix
